// ------------------------------------------------------------------------------------
// <copyright file="EstadosBL.cs" company="Universidad Distrital">
// Copyright (c) Universidad Distrital. All rights reserved.
// </copyright>
// <author>Camilo Prieto</author>
// ------------------------------------------------------------------------------------
namespace Biblioteca.Negocio.Clases.BL
{
	using System;
	using System.Collections.Generic;
	using System.Threading.Tasks;
	using Biblioteca.Datos.Clases.DAL.Repositorio;
	using Biblioteca.IC.Acciones.Repositorio;
	using Biblioteca.IC.DTO.Repositorio;
	using Biblioteca.IC.Recursos;
	using TransaccionalNet.EF.Clases;
	using UtilitariosNet.Clases.Comunes.Entidades;
	using Utilitarios.TransversalesNet.Clases.Comunes.Enumeraciones;

	/// <summary>
	/// Clase con las acciones de negocio de la entidad Estados
	/// </summary>
	public class EstadosBL : AccesoComunBL, IEstadosNegocioAccion
	{

		#region ATRIBUTOS
		/// <summary>
		/// Objeto para repositorio Estados
		/// <summary>
		/// Objeto para acciones Estados
		/// </summary>
		private Lazy<IEstadosRepositorioAccion> estadosRepositorioAccion;

		/// <summary>
		/// Objeto para entidad respuesta
		/// </summary>
		Respuesta<IEstadosDTO> respuesta;

		#endregion ATRIBUTOS
		#region CONSTRUCTORES
		/// <summary>
		/// Inicializa una nueva instancia de la clase EstadosBL
		/// </summary>
		/// <param name="argEstadosRepositorioAccion">Acciones entidad Estados</param>
		public EstadosBL(Lazy<IEstadosRepositorioAccion> argEstadosRepositorioAccion = null)
		{
			this.respuesta = new Respuesta<IEstadosDTO>();
			this.estadosRepositorioAccion = argEstadosRepositorioAccion ?? new Lazy<IEstadosRepositorioAccion>(() => new EstadosDAL());
		}

		#endregion CONSTRUCTORES
		#region METODOS PUBLICOS
		/// <summary>
		/// Metodo consultar lista estados
		/// </summary>
		/// <returns>Respuesta tipo Estados </returns>
		public async Task<Respuesta<IEstadosDTO>> ConsultarListaEstadosAsync()
		{

			return await this.EjecutarTransaccionBDAsync<Respuesta<IEstadosDTO>, EstadosBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.estadosRepositorioAccion.Value.ConsultarListaEstadosAsync();
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 return respuesta;

			});

		}
		/// <summary>
		/// Metodo consultar por llave estados
		/// </summary>
		/// <param name="estados">Entidad a consultar</param>
		/// <returns>Respuesta tipo Estados </returns>
		public async Task<Respuesta<IEstadosDTO>> ConsultarEstadosLlaveAsync(IEstadosDTO estados)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<IEstadosDTO>, EstadosBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.estadosRepositorioAccion.Value.ConsultarEstadosLlaveAsync(estados);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 return respuesta;

			});
		}

		/// <summary>
		/// Metodo editar estados
		/// </summary>
		/// <param name="estados">Entidad a editar</param>
		/// <returns>Respuesta tipo Estados </returns>
		public async Task<Respuesta<IEstadosDTO>> EditarEstadosAsync(IEstadosDTO estados)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<IEstadosDTO>, EstadosBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.estadosRepositorioAccion.Value.EditarEstadosAsync(estados);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 respuesta.Mensajes.Add(rcsMensajesComunes.MensajeCreacionEdicionExitosa);
				 return respuesta;

			});
		}


		/// <summary>
		/// Metodo eliminar estados
		/// </summary>
		/// <param name="estados">Entidad a eliminar</param>
		/// <returns>Respuesta tipo Estados </returns>
		public async Task<Respuesta<IEstadosDTO>> EliminarEstadosAsync(IEstadosDTO estados)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<IEstadosDTO>, EstadosBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.estadosRepositorioAccion.Value.EliminarEstadosAsync(estados);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 respuesta.Mensajes.Add(rcsMensajesComunes.MensajeEntidadEliminadaConExito);
				 return respuesta;

			});
		}

		/// <summary>
		/// Metodo guardar estados
		/// </summary>
		/// <param name="estados">Entidad a guardar</param>
		/// <returns>Respuesta tipo Estados </returns>
		public async Task<Respuesta<IEstadosDTO>> GuardarEstadosAsync(IEstadosDTO estados)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<IEstadosDTO>, EstadosBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.estadosRepositorioAccion.Value.GuardarEstadosAsync(estados);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 respuesta.Mensajes.Add(rcsMensajesComunes.MensajeCreacionEdicionExitosa);
				 return respuesta;

			});
		}

		/// <summary>
		/// Metodo Guardar lista estados
		/// </summary>
		/// <param name="listaEstados">Lista de entidades a guardar</param>
		/// <returns>Respuesta tipo Estados </returns>
		public async Task<Respuesta<IEstadosDTO>> GuardarListaEstadosAsync(List<IEstadosDTO> listaEstados)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<IEstadosDTO>, EstadosBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.estadosRepositorioAccion.Value.GuardarListaEstadosAsync(listaEstados);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 respuesta.Mensajes.Add(rcsMensajesComunes.MensajeCreacionEdicionExitosa);
				 return respuesta;

			});
		}
		#endregion METODOS PUBLICOS
		#region METODOS PRIVADOS
		#endregion METODOS PRIVADOS
	}
}

