// ------------------------------------------------------------------------------------
// <copyright file="EstadosController.cs" company="Universidad Distrital">
// Copyright (c) Universidad Distrital. All rights reserved.
// </copyright>
// <author>Camilo Prieto</author>
// ------------------------------------------------------------------------------------
namespace Biblioteca.API.Controllers
{
	using System;
	using System.Collections.Generic;
	using System.Threading.Tasks;
	using Utilitarios.APINet.Clases.Ejecucion;
	using UtilitariosNet.Clases.Comunes.Entidades;
	using UtilitariosNet.Clases.Mapeador.Extensiones;
	using Biblioteca.API.Models.Repositorio;
	using Biblioteca.IC.Acciones.Repositorio;
	using Biblioteca.Negocio.Clases.BL;
	using Microsoft.AspNetCore.Mvc;
	
	/// <summary>
	/// Clase con las capaciades Rest de la entidad Estados
	/// </summary>
	[Route("api/Estados")]
	public class EstadosController : AccesoComunAPI
	{


		/// <summary>
		/// Objeto para negocio Estados
		/// </summary>
		private Lazy<IEstadosNegocioAccion> negocioEstados;

		
		/// <summary>
		/// Inicializa una nueva instancia de la clase EstadosController
		/// </summary>
		public EstadosController()
		{
			this.negocioEstados = new Lazy<IEstadosNegocioAccion>(
										() => new EstadosBL());
		}
		
		/// <summary>
		/// Metodo consultar lista estados
		/// </summary>
		/// <returns>Respuesta tipo Estados </returns>
		[HttpGet]
		[Route("ConsultarListaEstados")]
		public async Task<Respuesta<Estados>> ConsultarListaEstados()
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Estados>>, EstadosController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Estados>>(await negocioEstados.Value.ConsultarListaEstadosAsync());
			});
		}
		
		/// <summary>
		/// Metodo consultar por llave estados
		/// </summary>
		/// <param name="estados">Entidad a consultar</param>
		/// <returns>Respuesta tipo Estados </returns>
		[HttpPost]
		[Route("ConsultarEstadosLlave")]
		public async Task<Respuesta<Estados>> ConsultarEstadosLlave(Estados estados)
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Estados>>, EstadosController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Estados>>(await negocioEstados.Value.ConsultarEstadosLlaveAsync(estados));
			});
		}
		
		/// <summary>
		/// Metodo guardar estados
		/// </summary>
		/// <param name="estados">Entidad a guardar</param>
		/// <returns>Respuesta tipo Estados </returns>
		[HttpPost]
		[Route("GuardarEstados")]
		public async Task<Respuesta<Estados>> GuardarEstados(Estados estados)
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Estados>>, EstadosController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Estados>>(await negocioEstados.Value.GuardarEstadosAsync(estados));
			});
		}
		
		/// <summary>
		/// Metodo editar estados
		/// </summary>
		/// <param name="estados">Entidad a editar</param>
		/// <returns>Respuesta tipo Estados </returns>
		[HttpPut]
		[Route("EditarEstados")]
		public async Task<Respuesta<Estados>> EditarEstados(Estados estados)
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Estados>>, EstadosController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Estados>>(await negocioEstados.Value.EditarEstadosAsync(estados));
			});
		}
		
		/// <summary>
		/// Metodo eliminar estados
		/// </summary>
		/// <param name="estados">Entidad a eliminar</param>
		/// <returns>Respuesta tipo Estados </returns>
		[HttpDelete]
		[Route("EliminarEstados")]
		public async Task<Respuesta<Estados>> EliminarEstados(Estados estados)
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Estados>>, EstadosController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Estados>>(await negocioEstados.Value.EliminarEstadosAsync(estados));
			});
		}
	}
}

